// VersionNumber.java
// $Id: VersionNumber.java,v 1.2 1999/11/24 10:25:44 bmahe Exp $
// (c) COPYRIGHT MIT, INRIA and Keio, 1999.
// Please first read the full copyright statement in file COPYRIGHT.html
package org.w3c.jwput;

/**
 * To compare two version number
 * @version $Revision: 1.2 $
 * @author  Benot Mah (bmahe@w3.org)
 */
public class VersionNumber {

    protected int major  = 0;
    protected int stable = 0;
    protected int minor  = 0;

    private int toInt(char c) {
	return ((int)c)-48;
    }

    /**
     * Compare version numbers.
     * @param vn a Version Number
     * @return true is the current Version Number is greater than the
     * given one.
     */
    public boolean greaterThan(VersionNumber vn) {
	return ((major > vn.major) ||
		(stable > vn.stable) ||
		(minor > vn.minor));
    }

    public String toString() {
	if (minor == 0)
	    return major+"."+stable;
	else
	    return major+"."+stable+"."+minor;
    }

    /**
     * Constructor
     * @param version the version number (eg: "1.0.1")
     */
    public VersionNumber(String version) 
	throws VersionFormatException
    {
	// "1.0.1" , "1.0" = "1.0.0"
	//length = 5 or 3
	int len = version.length();
	if (len == 3) {
	    major  = toInt(version.charAt(0));
	    stable = toInt(version.charAt(2));
	} else if (len == 5) {
	    major  = toInt(version.charAt(0));
	    stable = toInt(version.charAt(2));
	    minor  = toInt(version.charAt(4));
	} else {
	    throw new VersionFormatException(version);
	}
    }
}

