// JWGUI.java
// $Id: JWGUI.java,v 1.26 1999/11/24 10:10:52 bmahe Exp $
// (c) COPYRIGHT MIT, INRIA and Keio, 1999.
// Please first read the full copyright statement in file COPYRIGHT.html

package org.w3c.jwput.gui;

import java.io.*;

import java.net.*;

import java.util.Hashtable;
import java.util.Properties;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Cursor;
import java.awt.Font;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JMenu;
import javax.swing.JPanel;
import javax.swing.BorderFactory;
import javax.swing.UIManager;
import javax.swing.ProgressMonitorInputStream;
import javax.swing.ProgressMonitor;

import org.w3c.tools.widgets.Utilities;

import org.w3c.jigadmin.editors.MiniBrowser;

import org.w3c.util.XMLProperties;

import org.w3c.jwput.JWUI;
import org.w3c.jwput.JW;
import org.w3c.jwput.JWPut;
import org.w3c.jwput.JWGet;
import org.w3c.jwput.JWOptions;

/**
 * Winie's Graphical User Interface
 * @version $Revision: 1.26 $
 * @author  Benot Mah (bmahe@w3.org)
 */
public class JWGUI extends JPanel implements JWUI {

    public final static String URL_HISTORY_P  = "URL HISTORY";
    public final static String FILE_HISTORY_P = "FILE HISTORY";
    public final static String JWPUT_P        = "JWPUT";
    public final static String JWGET_P        = "JWGET";
    public final static String JWUI_P         = "JWUI";

    public final static String URL_HISTORY_FILE  = "winie/urlhistory";
    public final static String FILE_HISTORY_FILE = "winie/filehistory";
    public final static String LOG_FILE          = "winie/log";
    public final static String ERRLOG_FILE       = "winie/errlog";

    public final static int    HISTORY_SIZE   = 40;
    
    public static Properties jwprops = null;

    public static Icon winieIcon    = null;
    public static Icon messageIcon  = null;
    public static Icon errorIcon    = null;
    public static Icon questionIcon = null;

    //
    // listeners
    // 
    ActionListener actionListener = new ActionListener() {
	public void actionPerformed(ActionEvent evt) {
	    setCursor(Cursor.WAIT_CURSOR);
	    String command = evt.getActionCommand();
	    if (command.equals(JWMenu.DELETE_CMD)) {
		layout.show(cards, JWMenu.DELETE_CMD);
		status.clearStatus();
	    } else if (command.equals(JWMenu.LOAD_CMD)) {
		layout.show(cards, JWMenu.LOAD_CMD);
		status.clearStatus();
		state = LOAD_STATE;
	    } else if (command.equals(JWMenu.SAVE_CMD)) {
		layout.show(cards, JWMenu.SAVE_CMD);
		status.clearStatus();
		state = SAVE_STATE;
	    } else if (command.equals(JWMenu.QUIT_CMD)) {
		exit();
	    } else if (command.equals(JWMenu.PREF_CMD)) {
		showPreferences();
	    } else if (command.equals(JWMenu.HELP_CMD)) {
		try {
		    MiniBrowser.showDocumentationURL(JW.WINIE_DOC_URL,
						     "Winie Help");
		} catch (IOException ex) {
		    //nothing
		}
	    } else if (command.equals(JWMenu.ABOUT_CMD)) {
		String about[] = {
		    JW.BANNER,
		    "Version "+JW.VERSION,
		    "A program by Benot Mah (W3C)",
		    JW.WINIE_URL,
		    "(c) COPYRIGHT MIT, INRIA and Keio, 1999."
		};
		MessageDialog dialog = 
		    new MessageDialog("About Winie", about, 
				      getWinieIcon());
		dialog.showDialog(JWGUI.this);
	    } else if (command.equals(JWMenu.CONSOLE_CMD)) {
		showlogs();
	    } else if (command.equals(JWMenu.UPGRADE_CMD)) {
		(new Thread() {
		    public void run() {
			setCursor(Cursor.WAIT_CURSOR);
			jwget.upgradeWinie();
			setCursor(Cursor.DEFAULT_CURSOR);
		    }
		}).start();
	    }
	    setCursor(Cursor.DEFAULT_CURSOR);
	}
    };

    protected JWPut jwput = null;
    protected JWGet jwget = null;

    protected History urlHistory  = null;
    protected History fileHistory = null;

    protected Hashtable props = null;

    protected CardLayout layout = null;
    protected JFrame     frame  = null;
    protected JPanel     cards  = null;

    protected PasswordDialog pdialog = null;
    protected JWPreferences  prefs   = null;
    protected LogDialog      logD    = null;
    protected StatusBar      status  = null;

    protected OutputStream outlog     = null;
    protected OutputStream errlog     = null;
    protected File         logfile    = null;
    protected File         errlogfile = null;

    protected boolean showlogs   = false;
    protected boolean hidepopups = false;

    private final int SAVE_STATE = 0;
    private final int LOAD_STATE = 1;

    private int state = SAVE_STATE;

    public static void unBoldSpecificFonts() {
	Font f  = new Font("Dialog",Font.PLAIN,12); 
	Font fl = new Font("Dialog",Font.BOLD,12); 
  
	UIManager.put("Button.font",f); 
	UIManager.put("CheckBox.font",f); 
	UIManager.put("CheckBoxMenuItem.font",f); 
	UIManager.put("ComboBox.font",f); 
	UIManager.put("DesktopIcon.font",f); 
  
	UIManager.put("InternalFrame.font",f); 
	UIManager.put("InternalFrame.titleFont",f); 
	UIManager.put("Label.font",fl); 
	UIManager.put("Menu.font",f); 
	UIManager.put("MenuBar.font",f); 
  
	UIManager.put("MenuItem.font",f); 
	UIManager.put("ProgressBar.font",f); 
	UIManager.put("RadioButton.font",f); 
	UIManager.put("RadioButtonMenuItem.font",f); 
	UIManager.put("TabbedPane.font",f); 
  
	UIManager.put("TitledBorder.font",f); 
	UIManager.put("ToggleButton.font",f); 
	UIManager.put("ToolBar.font",f); 
    } 

    /**
     * Set The Cursor.
     * @param cursor The Cursor type
     */
    public void setCursor(int cursor) {
	frame.setCursor(Cursor.getPredefinedCursor(cursor));
    }

    /**
     * Constructor
     * @param the parent JFrame
     */
    public JWGUI(JFrame frame) {
	initialize();
	this.frame = frame;
	WindowAdapter closer = new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		((JFrame)e.getWindow()).dispose();
		exit();
	    }
	};
	frame.addWindowListener(closer);
	frame.setJMenuBar(this.getMenuBar());
	frame.setSize(580, 305);
	build(getProperties());
    }

    protected void initialize() {
	props = new Hashtable();
	String homedir = System.getProperty("user.home");
	File urlfile = new File(homedir, URL_HISTORY_FILE);
	File filefile = new File(homedir, FILE_HISTORY_FILE);
	logfile = new File(homedir, LOG_FILE);
	errlogfile = new File(homedir, ERRLOG_FILE);
	JW.check(urlfile);
	JW.check(filefile);
	JW.check(logfile);
	JW.check(errlogfile);
	this.urlHistory  = new History(HISTORY_SIZE, URL_HISTORY_P, urlfile);
	this.fileHistory = new History(HISTORY_SIZE, FILE_HISTORY_P, filefile);
	this.jwput = new JWPut(this, getJWProperties());
	this.jwget = new JWGet(this, getJWProperties());
	try {
	    props.put(URL_HISTORY_P, urlHistory.load());
	    props.put(FILE_HISTORY_P, fileHistory.load());
	    props.put(JWPUT_P, jwput);
	    props.put(JWGET_P, jwget);
	    props.put(JWUI_P, this);
	} catch (Exception ex) {
	    ex.printStackTrace();
	    displayError("Initialization error", ex.getMessage());
	}
    }

    protected Hashtable getProperties() {
	return props;
    }

    /**
     * Get the default user properties
     * @return a Property set
     */
    public static Properties getJWProperties() {
	if (jwprops == null) {
	    jwprops = new XMLProperties();
	    jwprops.setProperty(JWOptions.VERBOSE_P, "false");
	    jwprops.setProperty(JWOptions.PROMPT_P, "true");
	}
	return jwprops;
    }

    public void hidePopups() {
	this.hidepopups = true;
    }

    public void showPopups() {
	this.hidepopups = false;
    }

    /**
     * Show the log to the User (and still write in the log stream)
     */
    public void showlogs() {
	showlogs = true;
	if (logD == null) {
	    logD = new LogDialog("Console", this);
	} else {
	    logD.clean();
	}
	logD.showDialog(this);
    }

    /**
     * Hide the log to the User (and still write in the log stream)
     */
    public void hidelogs() {
	showlogs = false;
    }


    protected void build(Hashtable props) {
	// cards
	cards = new JPanel(layout = new CardLayout(5,5));
	cards.setBorder(BorderFactory.createLoweredBevelBorder());
 	cards.add(getDeleteComponent(props), JWMenu.DELETE_CMD);
 	cards.add(getLoadComponent(props), JWMenu.LOAD_CMD);
	cards.add(getSaveComponent(props), JWMenu.SAVE_CMD);
	//status panel
	status = getStatusBar();
	setLayout(new BorderLayout());
	add(cards, "Center");
	add(status, "South");
 	layout.show(cards, JWMenu.SAVE_CMD);
    }
    
    protected StatusBar getStatusBar() {
	Icon icons[] = { 
	    getIcon("/org/w3c/jwput/images/upload.gif"),
	    getIcon("/org/w3c/jwput/images/download.gif"),
	    getIcon("/org/w3c/jwput/images/delete.gif") 
	};
	String commands[] = {
	    JWMenu.SAVE_CMD,
	    JWMenu.LOAD_CMD,
	    JWMenu.DELETE_CMD
	};
	status = new StatusBar(icons, commands, commands, JW.BANNER);
	status.addActionListener(actionListener);
	return status;
    }
    
    protected void switchUploadDownload() {
	if (state == SAVE_STATE) {
	    layout.show(cards, JWMenu.LOAD_CMD);
	    state = LOAD_STATE;
	    status.clearStatus();
	} else {
	    layout.show(cards, JWMenu.SAVE_CMD);
	    state = SAVE_STATE;
	    status.clearStatus();
	}
    }

    protected Icon getWinieIcon() {
	if (winieIcon == null) {
	    winieIcon = getIcon("/org/w3c/jwput/images/winie.gif");
	}
	return winieIcon;
    }

    protected Icon getMsgIcon() {
	if (messageIcon == null) {
	    messageIcon = getIcon("/org/w3c/jwput/images/winie-message.gif");
	}
	return messageIcon;
    }

    protected Icon getErrorIcon() {
	if (errorIcon == null) {
	    errorIcon = getIcon("/org/w3c/jwput/images/winie-error.gif");
	}
	return errorIcon;
    }

    protected Icon getQuestionIcon() {
	if (questionIcon == null) {
	    questionIcon = getIcon("/org/w3c/jwput/images/winie-question.gif");
	}
	return questionIcon;
    }

    protected Icon getIcon(String name) {
	URL location = getClass().getResource(name);
	return new ImageIcon(location);
    }

    protected JComponent getDeleteComponent(Hashtable props) {
	return new DeleteComponent(props);
    }

    protected JComponent getLoadComponent(Hashtable props) {
	return new LoadComponent(props);
    }

    protected JComponent getSaveComponent(Hashtable props) {
	return new SaveComponent(props);
    }

    /**
     * Get the MenuBar.
     * @return a JMenuBar instance.
     */
    protected JMenuBar getMenuBar() {
	return new JWMenu(actionListener);
    }

    protected void exit() {
	try {
	    urlHistory.save();
	    fileHistory.save();
	    if (outlog != null)
		outlog.close();
	    if (errlog != null)
		errlog.close();
	    Properties props = null;
	    if (prefs != null)
		props = prefs.getPreferences();
	    else
		props = jwput.getUserProperties();
	    savePreferences(props);
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
	System.exit(0);
    }

    protected void showPreferences() {
	if (prefs == null)
	    prefs = new JWPreferences(jwput.getUserProperties());
	int returnVal = prefs.showDialog(this);
	if (returnVal == JWPreferences.APPROVE_OPTION) {
	    //update jwput and jwget
	    Properties userprefs = prefs.getPreferences();
	    jwput.updateProperties(userprefs);
	    jwget.updateProperties(userprefs);
	    savePreferences(userprefs);
	}
    }

    protected void savePreferences(Properties prefs) {
	try {
	    JW.savePreferences(prefs);
	} catch (IOException ex) {
	    displayError("Unable to save preferences.");
	}
    }

    //
    // JWUI implementation
    //

    /**
     * Get a username and a password
     * @param firstime is is the first time we ask for a password?
     * @return a String array containing { user, password }
     */
    public String[] getAuthentication(boolean firstime) {
	String title = (firstime) 
	    ? "Authorization required" 
	    : "Invalid username/password";
	pdialog = new PasswordDialog(title);
	int returnVal = pdialog.showDialog(this);
	if (returnVal ==  PasswordDialog.APPROVE_OPTION) {
	    String value[] = { pdialog.getUser(),
			       pdialog.getPassword() };
	    return value;
	}
	return null;
    }

    /**
     * Get a String from the user.
     * @param message the message to display
     * @return a String instance
     */
    public String getString(String message) {
	String array[] = { message };
	return getString(array);
    }

    /**
     * Get a String from the user.
     * @param message the message to display
     * @return a String instance
     */
    public String getString(String message[]) {
	StringDialog dialog = new StringDialog("Question", message);
	dialog.showDialog(this);
	return dialog.getString();
    }

    /**
     * Get a File from the user.
     * @param message the message to display
     * @return a String instance
     */
    public String getFile(String title, String message) {
	String msg[] = { message };
	return getFile(title, msg);
    }

    /**
     * Get a File from the user.
     * @param message the message to display
     * @return a String instance
     */
    public String getFile(String title, String message[]) {
	FileDialog dialog = new FileDialog(title, message);
	dialog.showDialog(this);
	return dialog.getFile();
    }

    /**
     * Ask a question to the user.
     * @param question the Question
     * return a boolean (true = yes, false = no)
     */
    public boolean askQuestion(String question) {
	String array[] = { question };
	return askQuestion((String[])null, array);
    }


    /**
     * Ask a question to the user.
     * @param message a message to display before the question
     * @param question the Question
     * return a boolean (true = yes, false = no)
     */
    public boolean askQuestion(String message, String question) {
	String array[] = { message };
	String array2[] = { question };
	return askQuestion(array, array2);
    }

    /**
     * Ask a question to the user.
     * @param question the Question
     * @param message a message to display before the question (several lines)
     * @return a boolean (true = yes, false = no)
     */
    public boolean askQuestion(String message[], String question) {
	String array[] = { question };
	return askQuestion(array, question);
    }

    /**
     * Ask a question to the user.
     * @param question the Question (several lines)
     * @param message a message to display before the question (several lines)
     * @return a boolean (true = yes, false = no)
     */
    public boolean askQuestion(String message[], String question[]) {
	QuestionDialog dialog = 
	    new QuestionDialog("Question", message, question, 
			       getQuestionIcon());
	return (dialog.showDialog(this) == QuestionDialog.APPROVE_OPTION);
    }

    /**
     * Ask a question to the user.
     * @param question the Question (several lines)
     * @param message a message to display before the question (several lines)
     * @return APPROVE_OPTION, CANCEL_OPTION, ABORT_OPTION
     */
    public int ask(String message[], String question[]) {
	QuestionDialog dialog = 
	    new QuestionDialog("Question", message, question, 
			       getQuestionIcon());
	return dialog.showDialog(this);
    }

    /**
     * Display an error message
     * @param msg the error message
     */
    public void displayError(String msg) {
	String array[] = { msg };
	displayError(array, "Error");
    }

    /**
     * Display an error message
     * @param msg the error message
     */
    public void displayError(String msg, String title) {
	String array[] = { msg };
	displayError(array, title);
    }

    /**
     * Display a error message (on several lines)
     * @param msg the message
     */
    public void displayError(String msg[], String title) {
	log(msg);
	MessageDialog dialog = 
	    new MessageDialog(title, msg, getErrorIcon());
	dialog.showDialog(this);
    }

    /**
     * Display a message
     * @param msg the message
     */
    public void displayMessage(String msg) {
	displayMessage(msg, "Message");
    }

    /**
     * Display a message
     * @param msg the message
     * @param title the dialog title
     */
    public void displayMessage(String msg, String title) {
	String array[] = { msg };
	displayMessage(array, title);
    }

    /**
     * Display a message (on several lines)
     * @param msg the message
     * @param title the dialog title
     */
    public void displayMessage(String msg[], String title) {
	displayLog(msg);
	if (! hidepopups) {
	    MessageDialog dialog = new MessageDialog(title, msg, getMsgIcon());
	    dialog.showDialog(this);
	}
    }

    /**
     * Display a message (on several lines)
     * @param msg the message
     * @param title the dialog title
     * @param icon the Icon to display
     */
    public void displayMessage(String msg[], String title, Icon icon) {
	displayLog(msg);
	if (! hidepopups) {
	    MessageDialog dialog = new MessageDialog(title, msg, icon);
	    dialog.showDialog(this);
	}
    }

    protected void displayLog(String log[]) {
	for (int i = 0 ; i < log.length ; i++) {
	    if (showlogs)
		logD.println(log[i]);
	    status.setStatus(log[i]);
	}
    }

    protected void displayLog(String log) {
	if (showlogs)
	    logD.println(log);
	status.setStatus(log);
    }
    
    protected void log(String log[]) {
	displayLog(log);
	try {
	    for (int i = 0 ; i < log.length ; i++) {
		getLogStream().write(log[i].getBytes());
		getLogStream().write('\n');
	    }
	} catch (IOException ex) {
	    ex.printStackTrace();
	}
    }

    /**
     * Log a string
     * @param the String to log
     */
    public void log(String log) {
	if (showlogs) {
	    displayLog(log);
	}
	try {
	    getLogStream().write(log.getBytes());
	    getLogStream().write('\n');
	} catch (IOException ex) {
	    ex.printStackTrace();
	}
    }

    /**
     * Log an error
     * @param the String to log
     */
    public void errlog(String log) {
	if (showlogs) {
	    displayLog(log);
	}
	try {
	    getErrlogStream().write(log.getBytes());
	    getErrlogStream().write('\n');
	} catch (IOException ex) {
	    ex.printStackTrace();
	}
    }

    /**
     * Flush the logs (log and errlog)
     */
    public void sync() {
	try {
	    getLogStream().flush();
	    getErrlogStream().flush();
	} catch (IOException ex) {
	    //nothing it's just log
	}
    }

    /**
     * Get a Stream on the log.
     * @return an OutputStream
     */
    public OutputStream getLogStream() {
	if (outlog == null) {
	    try {
		outlog = new BufferedOutputStream(
			new FileOutputStream(logfile.getAbsolutePath(), true));
	    } catch (IOException ex) {
		ex.printStackTrace();
	    }
	}
	return outlog;
    }

    /**
     * Get a Stream on the errlog.
     * @return an OutputStream
     */
    public OutputStream getErrlogStream() {
	if (errlog == null) {
	    try {
		errlog = new BufferedOutputStream(
			new FileOutputStream(errlogfile.getAbsolutePath(),
					     true));
	    } catch (IOException ex) {
		ex.printStackTrace();
	    }
	}
	return errlog;
    }

    /**
     * Get a monitored inputstream
     * @param in the inputstream to monitor
     * @param message the message to display during monitoring
     * @param length the number of byte to be read
     * @return the monitored inputstream
     */
    public InputStream getMonitoredInputStream(InputStream in,
					       String message,
					       int length)
    {
	ProgressMonitorInputStream min =
	    new ProgressMonitorInputStream(this, message, in);
 	ProgressMonitor monitor = min.getProgressMonitor();
 	monitor.setMinimum(0);
 	monitor.setMaximum(length);
	monitor.setMillisToPopup(0);
	monitor.setMillisToDecideToPopup(0);
	return min;
    }

    /**
     * Main
     */
    public static void main(String[] args) {
	unBoldSpecificFonts();
	JFrame frame = new JFrame("Winie HTTP/1.1");
 	JWGUI  gui   = new JWGUI(frame);
	frame.getContentPane().add(gui, BorderLayout.CENTER);
 	frame.setVisible(true);
    }

}
