// JWPreferences.java
// $Id: JWPreferences.java,v 1.12 1999/11/24 10:10:53 bmahe Exp $
// (c) COPYRIGHT MIT, INRIA and Keio, 1999.
// Please first read the full copyright statement in file COPYRIGHT.html
 
package org.w3c.jwput.gui;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Properties;
import java.util.Enumeration;
import java.util.StringTokenizer;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Insets;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListModel;
import javax.swing.AbstractListModel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.BorderFactory;
import javax.swing.SwingUtilities;
import javax.swing.JTextField;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

import org.w3c.tools.widgets.Utilities;
import org.w3c.www.mime.Utils;
import org.w3c.jwput.JWOptions;

/**
 * Dialog allowing user to set his preferences
 * @version $Revision: 1.12 $
 * @author  Benot Mah (bmahe@w3.org)
 */
public class JWPreferences extends JPanel implements JWDialog, JWOptions {
    
    public static final String TITLE = "Preferences";

    public static final String OK_B     = "  Ok  ";
    public static final String CANCEL_B = "Cancel";

    public static final String PROXY_P      = "Proxy";
    public static final String LANGUAGES_P  = "Languages";
    public static final String AUTH_P       = "Authentication";
    public static final String CONNECTION_P = "Connection";
    public static final String MIME_P       = "Mime";

    public static final String DIRECT_P = "direct";
    public static final String PROXYH_P = "proxyh";

    protected Properties prefs = null;

    protected JDialog    dialog = null;
    protected JList      list   = null;
    protected CardLayout layout = null;
    protected JPanel     cards  = null;

    protected int returnVal = ERROR_OPTION;

    protected ProxyPrefs proxyP = null;
    protected LangPrefs  langP  = null;
    protected ConnPrefs  connP  = null;
    protected MimePrefs  mimeP  = null;
    protected AuthPrefs  authP  = null;

    //
    // Inner Classes
    //

    class ProxyPrefs extends JPanel {

	JRadioButton direct = null;
	JRadioButton proxy  = null;
	JTextField   thost  = null;
	JTextField   tport  = null;
	JTextField   tshost = null;
	JTextField   tsport = null;

	String getHost() {
	    return thost.getText();
	}

	String getPort() {
	    return tport.getText();
	}

	String getSHost() {
	    return tshost.getText();
	}

	String getSPort() {
	    return tsport.getText();
	}

	boolean useProxy() {
	    return direct.isSelected();
	}

	ProxyPrefs() {
	    setLayout(new BorderLayout());
	    setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));

	    //
	    // Radio buttons
	    //
	    ActionListener al = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    String command = e.getActionCommand();
		    if (command.equals(PROXYH_P)) {
			thost.setEditable(true);
			tport.setEditable(true);
			tshost.setEditable(true);
			tsport.setEditable(true);
		    } else if (command.equals(DIRECT_P)) {
			thost.setEditable(false);
			tport.setEditable(false);
			tshost.setEditable(false);
			tsport.setEditable(false);
		    }
		}
	    };

	    direct = new JRadioButton("Direct connection to the Internet");
	    direct.setActionCommand(DIRECT_P);
	    direct.addActionListener(al);

	    proxy = new JRadioButton("Manual proxy configuration");
	    proxy.setActionCommand(PROXYH_P);
	    proxy.addActionListener(al);

	    JPanel pbuttons = new JPanel(new GridLayout(2,1));
	    pbuttons.add(direct);
	    pbuttons.add(proxy);

	    ButtonGroup group = new ButtonGroup();
	    group.add(direct);
	    group.add(proxy);

	    //
	    // Proxy
	    //
	    JLabel host = new JLabel("Proxy Host");
	    JLabel port = new JLabel("Port");
	    thost = new JTextField(10);
	    thost.setBorder(BorderFactory.createLoweredBevelBorder());
	    tport = new JTextField(5);
	    tport.setBorder(BorderFactory.createLoweredBevelBorder());
	
	    GridBagLayout gbl = new GridBagLayout();
	    GridBagConstraints gbc = new GridBagConstraints();
	    gbc.fill = GridBagConstraints.HORIZONTAL;
	    gbc.weightx = 0.9;
	    gbc.weighty = 0;
	    gbc.insets = Utilities.insets4;

	    JPanel pproxy = new JPanel(gbl);
	    pproxy.setBorder(BorderFactory.createEmptyBorder(0, 10, 0, 0));

	    gbc.gridwidth = 1;
	    gbl.setConstraints(host, gbc);
	    pproxy.add(host);

	    gbc.gridwidth = 1;
	    gbl.setConstraints(thost, gbc);
	    pproxy.add(thost);

	    gbc.gridwidth = 1;
	    gbl.setConstraints(port, gbc);
	    pproxy.add(port);

	    gbc.gridwidth = GridBagConstraints.REMAINDER;;
	    gbl.setConstraints(tport, gbc);
	    pproxy.add(tport);

	    //
	    // SOCK Proxy
	    //

	    JLabel shost = new JLabel("Socks Host");
	    JLabel sport = new JLabel("Port");
	    tshost = new JTextField(10);
	    tshost.setBorder(BorderFactory.createLoweredBevelBorder());
	    tsport = new JTextField(5);
	    tsport.setBorder(BorderFactory.createLoweredBevelBorder());
	
	    gbc.gridwidth = 1;
	    gbl.setConstraints(shost, gbc);
	    pproxy.add(shost);

	    gbc.gridwidth = 1;
	    gbl.setConstraints(tshost, gbc);
	    pproxy.add(tshost);

	    gbc.gridwidth = 1;
	    gbl.setConstraints(sport, gbc);
	    pproxy.add(sport);

	    gbc.gridwidth = GridBagConstraints.REMAINDER;;
	    gbl.setConstraints(tsport, gbc);
	    pproxy.add(tsport);

	    //
	    JPanel pradio = new JPanel(new GridLayout(2,1));
	    pradio.setBorder(BorderFactory.createEmptyBorder(10,10,0,0));
	    pradio.add(pbuttons);
	    pradio.add(pproxy);

	    JPanel panel = new JPanel(new BorderLayout());
	    panel.add(pradio, "North");

	    JLabel l1 = new JLabel("Configure Proxy to access the Internet.");
	    l1.setBorder(BorderFactory.createEmptyBorder(0, 10, 0, 0));

	    JPanel panel1 = new JPanel(new BorderLayout());
	    panel1.setBorder(BorderFactory.createTitledBorder(PROXY_P));
	    panel1.add(l1, "North");
	    panel1.add(panel, "Center");

	    add(panel1);
	    initialize();
	}

	void initialize() {
	    String proxyset = prefs.getProperty(PROXY_SET_P);
	    proxyset = (proxyset == null) ? FALSE_P : proxyset;
	    if (proxyset.equalsIgnoreCase(FALSE_P)) {
		direct.setSelected(true);
		thost.setEditable(false);
		tport.setEditable(false);
		tshost.setEditable(false);
		tsport.setEditable(false);
	    } else {
		proxy.setSelected(true);
	    }
	    String host = prefs.getProperty(PROXY_HOST_P);
	    String port = prefs.getProperty(PROXY_PORT_P);
	    if (host != null)
		thost.setText(host);
	    if (port != null)
		tport.setText(port);
	    String shost = prefs.getProperty(SOCKS_HOST_P);
	    String sport = prefs.getProperty(SOCKS_PORT_P);
	    if (shost != null)
		tshost.setText(shost);
	    if (sport != null)
		tsport.setText(sport);
	    else
		tsport.setText("1080");

	}

	void save() {
	    boolean no_proxy = direct.isSelected();
	    prefs.setProperty(PROXY_SET_P, (no_proxy  ? FALSE_P : TRUE_P));
	    prefs.setProperty(PROXY_PORT_P, tport.getText());
	    prefs.setProperty(PROXY_HOST_P, thost.getText());
	    prefs.setProperty(SOCKS_HOST_P, tshost.getText());
	    prefs.setProperty(SOCKS_PORT_P, tsport.getText());
	}
    }

    class LangModel extends AbstractListModel {
	
	Vector langs = null;

	public int getSize() { 
	    return langs.size();
	}
		
	public Object getElementAt(int index) { 
	    return langs.elementAt(index);
	}

	public void insertElement(String lang, int index) {
	    langs.insertElementAt(lang, index);
	    fireIntervalAdded(this, index, index);
	}

	public void removeElement(int index) {
	    langs.removeElementAt(index);
	    fireIntervalRemoved(this, index, index);
	}

	/**
	 * index2 = index1+1
	 */
	public void switchElements(int index1, int index2) {
	    if (index1+1 != index2)
		return;
	    Object element = langs.elementAt(index1);
	    langs.setElementAt(langs.elementAt(index2), index1);
	    langs.setElementAt(element, index2);
	    fireContentsChanged(this, index1, index2);
	}

	LangModel() {
	    langs = new Vector(10);
	}

	LangModel(Vector langs) {
	    this.langs = langs;
	}

	LangModel(String langarray[]) {
	    int len = langarray.length;
	    langs = new Vector(len);
	    for (int i = 0; i < len; i++) {
		langs.addElement(langarray[i]);
	    }
	}
    }

    class LangPrefs extends JPanel {
	
	final String ADD_P  = "Add...";
	final String UP_P   = "Up";
	final String DOWN_P = "Down";
	final String DEL_P  = "Delete";

	LanguageDialog dialog = null;

	JList     langs = null;
	LangModel model = null;

	LangPrefs() {
	    ActionListener al = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    String command = e.getActionCommand();
		    if (command.equals(ADD_P)) {
			addLanguage();
		    } else if (command.equals(UP_P)) {
			int idx = langs.getSelectedIndex();
			if (idx > 0) {
			    model.switchElements(idx-1, idx);
			    langs.setSelectedIndex(idx-1);
			}
		    } else if (command.equals(DOWN_P)) {
			int idx = langs.getSelectedIndex();
			if (idx > -1 && idx < model.getSize()-1) {
			    model.switchElements(idx, idx+1);
			    langs.setSelectedIndex(idx+1);
			}
		    } else if (command.equals(DEL_P)) {
			int idx = langs.getSelectedIndex();
			if (idx != -1)
			    model.removeElement(idx);
		    }
		}
	    };

	    setLayout(new BorderLayout());
	    setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));

	    JLabel l1 = 
		new JLabel("Choose the language(s) you prefer, sometimes");
	    JLabel l2 =
		new JLabel("web pages are availables in several languages.");

	    JPanel plines = new JPanel(new GridLayout(2,1));
	    plines.setBorder(BorderFactory.createEmptyBorder(0, 10, 0, 0));
	    plines.add(l1);
	    plines.add(l2);

	    langs = new JList();
	    langs.setBorder(BorderFactory.createLoweredBevelBorder());
	    langs.setFixedCellWidth(100);
	    //langs.setSelectedIndex(0);

	    JPanel listp = new JPanel(new BorderLayout());
	    listp.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
	    listp.add(langs, "Center");

	    JButton add  = new JButton(ADD_P);
	    JButton up   = new JButton(UP_P);
	    JButton down = new JButton(DOWN_P);
	    JButton del = new JButton(DEL_P);
	    add.addActionListener(al);
	    up.addActionListener(al);
	    down.addActionListener(al);
	    del.addActionListener(al);

	    JPanel movep = new JPanel(new GridLayout(5,1));
	    movep.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
	    movep.add(add);
	    movep.add(del);
	    movep.add(new JLabel("Move..."));
	    movep.add(up);
	    movep.add(down);

	    JPanel mp = new JPanel();
	    mp.add(movep);

	    JPanel p1 = new JPanel(new BorderLayout());
	    p1.add(listp, "Center");
	    p1.add(mp, "East");

	    JPanel titled = new JPanel(new BorderLayout());
	    titled.setBorder(BorderFactory.createTitledBorder(LANGUAGES_P));
	    titled.add(plines, "North");
	    titled.add(p1, "Center");

	    add(titled, "Center");

	    initialize();
	}

	void addLanguage() {
	    if (dialog == null)
		dialog = new LanguageDialog("Add Language");
	    int returnValue = dialog.showDialog(this);
	    if (returnValue == LanguageDialog.APPROVE_OPTION) {
		model.insertElement(dialog.getLanguage(), 
				    langs.getSelectedIndex()+1);
	    }
	}

	void initialize() {
	    String acceptlang = prefs.getProperty(ACCEPT_LANGUAGE_P);
	    if (acceptlang != null) {
		StringTokenizer st = new StringTokenizer(acceptlang, ",");
		Vector v = new Vector();
		while (st.hasMoreTokens())
		    v.addElement(Languages.getLanguage(st.nextToken()));
		this.model = new LangModel(v);
	    } else {
		this.model = new LangModel();
	    }
	    langs.setModel(model);
	}

	void save() {
	    String acceptlang = "";
	    ListModel lmodel = langs.getModel();
	    for (int i = 0 ; i < lmodel.getSize() ; i++) {
		if (i != 0)
		    acceptlang += ",";
		acceptlang += 
		    Languages.getAbbrev((String)lmodel.getElementAt(i));
	    }
	    prefs.setProperty(ACCEPT_LANGUAGE_P, acceptlang);
	}
    }

    class AuthPrefs extends JPanel {
	JTextField     userT   = null;
	JPasswordField passwdT = null;
	AuthPrefs() {
	    setBorder(BorderFactory.createTitledBorder(AUTH_P));
	    GridBagLayout gbl = new GridBagLayout();
	    GridBagConstraints gbc = new GridBagConstraints();
	    GridBagLayout mgbl = new GridBagLayout();
	    GridBagConstraints mgbc = new GridBagConstraints();
	    JLabel l; JButton b;
	    JPanel p = new JPanel(gbl);

	    userT   = new JTextField(10);
	    userT.setBorder(BorderFactory.createLoweredBevelBorder());
	    passwdT = new JPasswordField(10);
	    passwdT.setBorder(BorderFactory.createLoweredBevelBorder());

	    gbc.fill = GridBagConstraints.HORIZONTAL;
	    gbc.weightx = 0;
	    gbc.weighty = 0;
	    mgbc.fill = GridBagConstraints.NONE;
	    mgbc.weightx = 0;
	    mgbc.weighty = 0;
	    mgbc.insets = new Insets(14, 10, 16, 5);
	    setLayout(mgbl);

	    l = new JLabel("User name: ", JLabel.RIGHT);
	    gbc.gridwidth = 1;
	    gbc.insets = new Insets(14, 0, 0, 0);
	    gbl.setConstraints(l, gbc);
	    p.add(l);
	    gbc.gridwidth = GridBagConstraints.REMAINDER;
	    gbl.setConstraints(userT, gbc);
	    p.add(userT);

	    l = new JLabel("Password: ", JLabel.RIGHT);
	    gbc.gridwidth = 1;
	    gbl.setConstraints(l, gbc);
	    p.add(l);
	    gbc.gridwidth = GridBagConstraints.REMAINDER;
	    gbl.setConstraints(passwdT, gbc);
	    p.add(passwdT);
	    mgbc.gridwidth = GridBagConstraints.REMAINDER;
	    mgbl.setConstraints(p, mgbc);
	    add(p);

	    initialize();
	}

	void initialize() {
	    String user   = prefs.getProperty(USER_P);
	    if (user != null)
		userT.setText(user);
	    String passwd = prefs.getProperty(PASSWORD_P);
	    if (passwd != null)
		passwdT.setText(passwd);
	}

	void save() {
	    prefs.setProperty(USER_P, userT.getText());
	    prefs.setProperty(PASSWORD_P, new String(passwdT.getPassword()));
	}
    }

    class ConnPrefs extends JPanel {

	JTextField retriesT = null;

	ConnPrefs() {
	    setBorder(BorderFactory.createTitledBorder(CONNECTION_P));
	    GridBagLayout gbl = new GridBagLayout();
	    GridBagConstraints gbc = new GridBagConstraints();
	    GridBagLayout mgbl = new GridBagLayout();
	    GridBagConstraints mgbc = new GridBagConstraints();
	    JLabel l; JButton b;
	    JPanel p = new JPanel(gbl);

	    retriesT = new JTextField(4);
	    retriesT.setBorder(BorderFactory.createLoweredBevelBorder());

	    gbc.fill = GridBagConstraints.HORIZONTAL;
	    gbc.weightx = 0;
	    gbc.weighty = 0;
	    mgbc.fill = GridBagConstraints.NONE;
	    mgbc.weightx = 0;
	    mgbc.weighty = 0;
	    mgbc.insets = new Insets(16, 10, 16, 5);
	    setLayout(mgbl);

	    JPanel lines = new JPanel(new GridLayout(2,1));
	    l = new JLabel("Enter the number of retries:");
	    lines.add(l);
	    l = new JLabel("(after a connection closed)");
 	    lines.add(l);
	    lines.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 20));

	    gbc.gridwidth = 1;
	    gbl.setConstraints(lines, gbc);
	    p.add(lines);
	    gbc.gridwidth = GridBagConstraints.REMAINDER;
	    gbl.setConstraints(retriesT, gbc);
	    p.add(retriesT);
	    
	    add(p);

	    initialize();
	}

	void initialize() {
	    String tries = prefs.getProperty(TRIES_P);
	    if (tries == null)
		tries = String.valueOf(DEF_TRIES);
	    retriesT.setText(tries);
	}

	void save() {
	    prefs.setProperty(TRIES_P, retriesT.getText());
	}
    }

    class MimePrefs extends JPanel implements ItemListener, ActionListener {

	Hashtable toadd = null;
	Vector    todel = null;
	JComboBox exts  = null;
	JComboBox mimes = null;
	String    ADD_P = "add";
	String    DEL_P = "del";

	MimePrefs() {
	    JLabel l1 = null;
	    JLabel l2 = null;
	    JLabel l3 = null;
	    JLabel l4 = null;

	    setBorder(BorderFactory.createTitledBorder(MIME_P));
	    JPanel extp = new JPanel();
	    
	    exts = new JComboBox();
	    exts.setEditable(true);
	    exts.addItemListener(this);
	    exts.setAlignmentX(LEFT_ALIGNMENT);

	    l1 = new JLabel("Enter the file extension here");
	    l2 = new JLabel("or select one from the list");
	    l1.setAlignmentX(LEFT_ALIGNMENT);
	    l2.setAlignmentX(LEFT_ALIGNMENT);

	    extp.setLayout(new BoxLayout(extp, BoxLayout.Y_AXIS));
	    extp.add(l1);
	    extp.add(l2);
	    extp.add(new JLabel(" "));
	    extp.add(exts);

	    JPanel mimep = new JPanel();
	    
	    mimes = new JComboBox(Utils.getKnownMimeTypesinVector());
	    mimes.setEditable(true);
	    mimes.setAlignmentX(LEFT_ALIGNMENT);

	    l1 = new JLabel("Enter the mime type here or");
	    l2 = new JLabel("select one from the list");
	    l1.setAlignmentX(LEFT_ALIGNMENT);
	    l2.setAlignmentX(LEFT_ALIGNMENT);

	    mimep.setLayout(new BoxLayout(mimep, BoxLayout.Y_AXIS));
	    mimep.add(l1);
	    mimep.add(l2);
	    mimep.add(new JLabel(" "));
	    mimep.add(mimes);

	    //text
	    l1 = new JLabel("Specify mime type for different file types.");
	    l2 = new JLabel("Here are described associations between file extensions ");
	    l3 = new JLabel("and mime types, Winie needs this information to upload ");
	    l4 = new JLabel("files on the web.");
	    l1.setAlignmentX(LEFT_ALIGNMENT);
	    l2.setAlignmentX(LEFT_ALIGNMENT);
	    l3.setAlignmentX(LEFT_ALIGNMENT);
	    l4.setAlignmentX(LEFT_ALIGNMENT);

	    JPanel textp = new JPanel();
	    textp.setBorder(BorderFactory.createEmptyBorder(0, 0, 10, 0));
	    textp.setLayout(new BoxLayout(textp, BoxLayout.Y_AXIS));
	    textp.add(l1);
	    textp.add(l2);
	    textp.add(l3);
	    textp.add(l4);

	    JPanel buttonp = new JPanel(new BorderLayout());

	    JButton delb = new JButton("Remove Association");
	    delb.setActionCommand(DEL_P);
	    delb.addActionListener(this);

	    JButton addb = new JButton("Save Association");
	    addb.setActionCommand(ADD_P);
	    addb.addActionListener(this);

	    buttonp.add("West", addb);
	    buttonp.add("East", delb);

	    JPanel p1 = new JPanel(new BorderLayout(10,10));
	    p1.add("West", extp);
	    p1.add("East", mimep);
	    
	    JPanel p2 = new JPanel(new BorderLayout());
	    p2.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	    p2.add("North", textp);
	    p2.add("Center", p1);
	    p2.add("South", buttonp);

	    setLayout(new BorderLayout());
	    add("Center",p2);

	    initialize();
	}

	void initialize() {
	    todel = new Vector();
	    toadd = new Hashtable();
	    exts.removeAllItems();
	    Enumeration enum = prefs.propertyNames();
	    int len = MIME_PREFIX.length();
	    Vector vmimes = Utils.getKnownMimeTypesinVector();
	    while (enum.hasMoreElements()) {
		String key = (String)enum.nextElement();
		if (key.startsWith(MIME_PREFIX)) {
		    exts.addItem(key.substring(len));
		    String mime = prefs.getProperty(key);
		    if (! vmimes.contains(mime)) {
			vmimes.addElement(mime);
			mimes.addItem(mime);
		    }
		}
	    }
	}

	void save() {
	    for (int i = 0 ; i < todel.size() ; i++)
		prefs.remove(todel.elementAt(i));
	    Enumeration enum = toadd.keys();
	    while (enum.hasMoreElements()) {
		String ext = (String)enum.nextElement();
		prefs.setProperty(ext, (String)toadd.get(ext));
	    }
	}

	private String getMimeType(String ext) {
	    String mext = MIME_PREFIX+ext;
	    String mime = (String)toadd.get(mext);
	    if (mime == null)
		mime = prefs.getProperty(mext);
	    return mime;
	}

	public void itemStateChanged(ItemEvent ev) {
	    if (ev.getSource() == exts) {
		String ext = (String)exts.getSelectedItem();
		String mime = getMimeType(ext);
		if (mime != null)
		    mimes.setSelectedItem(mime);
	    } 
	}

	public void actionPerformed(ActionEvent ev) {
	    if (ev.getActionCommand() == ADD_P) {
		String ext  = (String)exts.getSelectedItem();
		String mime = (String)mimes.getSelectedItem();
		toadd.put(MIME_PREFIX+ext, mime);
		exts.removeItem(ext);
		exts.addItem(ext);
		mimes.removeItem(mime);
		mimes.addItem(mime);
		exts.setSelectedItem(ext);
		mimes.setSelectedItem(mime);
	    } else if (ev.getActionCommand() == DEL_P) {
		String item  = (String)exts.getSelectedItem();
		String mitem = MIME_PREFIX+item;
		todel.addElement(mitem);
		toadd.remove(mitem);
		exts.removeItem(item);
		exts.setSelectedIndex(0);
		mimes.setSelectedItem(
				 getMimeType((String)exts.getSelectedItem()));
	    }
	}
    }

    //
    // JWPreferences
    //

    /**
     * Constructor
     * @param props the current preferences
     */
    public JWPreferences(Properties props) {
	this.prefs = props;
	build();
    }

    /**
     * Get the User preferences.
     * @return a Property Set
     */
    public Properties getPreferences() {
	return prefs;
    }

    protected void build() {
	//
	// list
	//
	ListSelectionListener lsl = new ListSelectionListener() {
	    public void valueChanged(ListSelectionEvent e) {
		displayPref();
	    }
	};

	String data[] = { AUTH_P, CONNECTION_P, LANGUAGES_P, MIME_P, PROXY_P };
	list = new JList(data);
	list.setBorder(BorderFactory.createLoweredBevelBorder());
	list.setFixedCellWidth(100);
	list.addListSelectionListener(lsl);

	JLabel categories = new JLabel("Categories", JLabel.CENTER);
	categories.setBorder(BorderFactory.createEmptyBorder(0, 0, 5, 0));

	JPanel plist = new JPanel(new BorderLayout());
	plist.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	plist.add(categories, "North");
	plist.add(list, "Center");

	//
	// prefs
	//
	
	cards = new JPanel(layout = new CardLayout(5, 5));
	cards.add(getAuthProps(), AUTH_P);
	cards.add(getConnProps(), CONNECTION_P);
	cards.add(getMimeProps(), MIME_P);
	cards.add(getLanguagesProps(), LANGUAGES_P);
	cards.add(getProxyProps(), PROXY_P);
	list.setSelectedIndex(0);

	//
	// Buttons
	//
	
	ActionListener al = new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (command.equals(OK_B)) {
		    save();
		} else if (command.equals(CANCEL_B)) {
		    quit();
		}
	    }
	};

	JButton okb = new JButton(OK_B);
	okb.addActionListener(al);

	JButton cancelb = new JButton(CANCEL_B);
	cancelb.addActionListener(al);

	JPanel pbuttons = new JPanel(new BorderLayout());
	pbuttons.setBorder(BorderFactory.createEmptyBorder(0,10,10,10));
	pbuttons.add(okb, "West");
	pbuttons.add(cancelb, "East");

	setLayout(new BorderLayout());
	add(cards, "Center");
	add(plist, "West");
	add(pbuttons, "South");

    }

    protected JComponent getProxyProps() {
	if (proxyP == null)
	    proxyP = new ProxyPrefs();
	return proxyP;
    }

    protected JComponent getLanguagesProps() {
	if (langP == null)
	    langP = new LangPrefs();
	return langP;
    }

    protected JComponent getAuthProps() {
	if (authP == null)
	    authP = new AuthPrefs();
	return authP;
    }

    protected JComponent getConnProps() {
	if (connP == null)
	    connP = new ConnPrefs();
	return connP;
    }

    protected JComponent getMimeProps() {
	if (mimeP == null)
	    mimeP = new MimePrefs();
	return mimeP;
    }

    protected void displayPref() {
	layout.show(cards, (String)list.getSelectedValue());
    }

    protected void save() {
	proxyP.save();
	langP.save();
	authP.save();
	mimeP.save();
	connP.save();
	returnVal = APPROVE_OPTION;
	dialog.setVisible(false);
    }

    protected void quit() {
	returnVal = CANCEL_OPTION;
	reset();
	dialog.setVisible(false);
    }

    protected void reset() {
	proxyP.initialize();
	langP.initialize();
	authP.initialize();
	mimeP.initialize();
	connP.initialize();
    }

    /**
     * Show this modal dialog
     * @param comp the parent component
     * @return CANCEL_OPTION, APPROVE_OPTION, ERROR_OPTION
     */
    public int showDialog(Component parent) {
	Frame frame = (parent instanceof Frame)
	    ? (Frame) parent
	    : (Frame)SwingUtilities.getAncestorOfClass(Frame.class, parent);

	dialog = new JDialog(frame, TITLE, true);
	Container contentPane = dialog.getContentPane();
	contentPane.setLayout(new BorderLayout());
	contentPane.add(this, BorderLayout.CENTER);
	
	dialog.pack();
	dialog.setSize(600,350);
	dialog.setLocationRelativeTo(parent);
        dialog.show();
	return returnVal;
    }
}
